# -*- coding: utf-8 -*-
import datetime
import json

from peewee import Model
from playhouse.shortcuts import model_to_dict


def encode_complex(obj):
    if isinstance(obj, datetime.datetime):
        return obj.__repr__()

    elif isinstance(obj, DictModel):
        return obj.to_dict()


class DictModel(Model):

    def to_data(self, recurse=True, backrefs=False,
                only=None, exclude=None,
                seen=None, extra_attrs=None,
                fields_from_query=None, max_depth=None,
                manytomany=False):
        """model to dict"""
        return model_to_dict(self,
                             recurse=recurse, backrefs=backrefs,
                             only=only, exclude=exclude,
                             seen=seen, extra_attrs=extra_attrs,
                             fields_from_query=fields_from_query,
                             max_depth=max_depth, manytomany=manytomany)

    def to_dict(self):
        """model to dict"""
        return dict(self.__data__)

    def __str__(self):
        """friendly for human"""
        return json.dumps(self, ensure_ascii=False, default=encode_complex)



# -*- coding: utf-8 -*-
import time
from functools import wraps
from typing import Iterable

from .model import DictModel
from .logger import logger


def to_dict(func):
    """
    model to dict , shallow convert
    """

    @wraps(func)
    def decorator(*args, **kwargs):
        logger.debug('decorator: to dict')

        result = func(*args, **kwargs)

        def _to_dict(item):
            if isinstance(item, DictModel):
                return item.to_dict()
            else:
                return item

        if isinstance(result, Iterable):
            return list(map(_to_dict, result))
        else:
            return _to_dict(result)

    return decorator


# 返回嵌套字典
def to_data(func=None, recurse=True, backrefs=False,
            only=None, exclude=None,
            seen=None, extra_attrs=None,
            fields_from_query=None, max_depth=1,
            manytomany=False):
    """model to dict , deep convert"""

    def inner(inner_func):
        @wraps(inner_func)
        def decorator(*func_args, **func_kwargs):
            logger.debug('decorator: to data')

            result = inner_func(*func_args, **func_kwargs)

            def _to_data(item):
                if isinstance(item, DictModel):
                    return item.to_data(
                        recurse=recurse, backrefs=backrefs,
                        only=only, exclude=exclude,
                        seen=seen, extra_attrs=extra_attrs,
                        fields_from_query=fields_from_query,
                        max_depth=max_depth, manytomany=manytomany)
                else:
                    return item

            if isinstance(result, Iterable):
                return list(map(_to_data, result))
            else:
                return _to_data(result)

        return decorator

    if func is None:
        return inner
    else:
        return inner(func)


def timer(func):
    """计时器"""
    @wraps(func)
    def wrapper(*args, **kwargs):
        start_time = time.time()

        ret = func(*args, **kwargs)

        end_time = time.time()
        logger.debug("time: %.2f s" % (end_time - start_time))

        return ret

    return wrapper



# -*- coding: utf-8 -*-
from peewee import MySQLDatabase, SENTINEL

from .decorator import timer


class TimerMySQLDatabase(MySQLDatabase):
    """
    打印sql执行时间
    see: https://github.com/coleifer/peewee/issues/2370
    """

    @timer
    def execute_sql(self, sql, params=None, commit=SENTINEL):
        return super().execute_sql(sql, params, commit)



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

import glob
import io
import os

from setuptools import setup, find_packages

"""
## 本地安装/卸载

python setup.py install
pip uninstall cator -y

## 测试安装/卸载
python setup.py develop
python setup.py develop -u
 
## 打包上传
先升级打包工具
pip install --upgrade setuptools wheel twine
打包
python setup.py sdist bdist_wheel
检查
twine check dist/*
上传pypi
twine upload dist/*
命令整合
rm -rf dist build *.egg-info \
&& python setup.py sdist bdist_wheel  \
&& twine check dist/* \
&& twine upload dist/* \
&& rm -rf dist build *.egg-info

## 下载测试
安装测试
pip3 install -U cator -i https://pypi.org/project

打包的用的setup必须引入
参考：
https://packaging.python.org/guides/making-a-pypi-friendly-readme/
"""

base_dir = os.path.dirname(os.path.abspath(__file__))

# 版本号
version_file = glob.glob("*/version.py", recursive=True)[0]

with io.open(version_file, 'rb') as f:
    version_var = {}
    exec(f.read(), version_var)
    VERSION = version_var['VERSION']

# 说明
with io.open("README.md", 'r', encoding='utf-8') as f:
    long_description = f.read()

# 依赖
with io.open("requirements.txt", 'r') as f:
    install_requires = f.read().split(os.linesep)

setup(
    name='peewee-ext',
    version=VERSION,
    description="peewee-ext is a peewee extension",

    keywords='peewee,mysql,pymysql,sqlite,dict',
    author='Peng Shiyu',
    author_email='pengshiyuyx@gmail.com',
    license='MIT',
    url="https://github.com/mouday/peewee-ext",

    long_description=long_description,
    long_description_content_type='text/markdown',

    # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Programming Language :: Python :: 3",
        'Programming Language :: Python :: 3.7'
    ],

    packages=find_packages(),
    include_package_data=True,
    zip_safe=True,
    install_requires=install_requires
)



# -*- coding: utf-8 -*-


from playhouse.db_url import schemes, register_database, connect as peewee_connect

from .database import TimerMySQLDatabase
from .model import DictModel
from .decorator import to_dict, to_data, timer

register_database(TimerMySQLDatabase, 'mysql+timer')


def connect(db_url=None, **kwargs):
    if db_url:
        return peewee_connect(db_url, **kwargs)
    else:
        scheme = kwargs.pop('scheme')
        return schemes[scheme](**kwargs)
